@@ -42,6 +42,7 @@
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
+#include "camel-mapi-sasl-krb.h"
 #include "camel-mapi-settings.h"
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-folder-summary.h"
@@ -1024,22 +1025,6 @@ mapi_store_finalize (GObject *object)
 	G_OBJECT_CLASS (camel_mapi_store_parent_class)->finalize (object);
 }
 
-static guint
-mapi_store_hash_folder_name (gconstpointer key)
-{
-	return g_str_hash(key);
-}
-
-static gint
-mapi_store_compare_folder_name (gconstpointer a,
-                          gconstpointer b)
-{
-	gconstpointer	aname = a;
-	gconstpointer	bname = b;
-
-	return g_str_equal(aname, bname);
-}
-
 static gboolean
 mapi_store_can_refresh_folder (CamelStore *store,
                                CamelFolderInfo *info,
@@ -1952,6 +1937,9 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	CamelServiceClass *service_class;
 	CamelStoreClass *store_class;
 
+	/* register MAPIKRB auth type */
+	CAMEL_TYPE_MAPI_SASL_KRB;
+
 	g_type_class_add_private (class, sizeof (CamelMapiStorePrivate));
 
 	object_class = G_OBJECT_CLASS (class);
@@ -1968,8 +1956,6 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	service_class->query_auth_types_sync = mapi_query_auth_types_sync;
 
 	store_class = CAMEL_STORE_CLASS (class);
-	store_class->hash_folder_name = mapi_store_hash_folder_name;
-	store_class->compare_folder_name = mapi_store_compare_folder_name;
 	store_class->can_refresh_folder = mapi_store_can_refresh_folder;
 	store_class->free_folder_info = camel_store_free_folder_info_full;
 	store_class->get_folder_sync = mapi_store_get_folder_sync;
@@ -2081,6 +2067,7 @@ mapi_connect_sync (CamelService *service,
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
 	CamelServiceConnectionStatus status;
 	CamelSession *session;
+	EMapiProfileData empd = { 0 };
 	uint64_t current_size = -1, receive_quota = -1, send_quota = -1;
 	gchar *name;
 
@@ -2104,7 +2091,9 @@ mapi_connect_sync (CamelService *service,
 	name = camel_service_get_name (service, TRUE);
 	camel_operation_push_message (cancellable, _("Connecting to '%s'"), name);
 
-	if (!camel_session_authenticate_sync (session, service, NULL, cancellable, error)) {
+	e_mapi_util_profiledata_from_settings (&empd, CAMEL_MAPI_SETTINGS (camel_service_get_settings (service)));
+
+	if (!camel_session_authenticate_sync (session, service, empd.krb_sso ? "MAPIKRB" : NULL, cancellable, error)) {
 		camel_operation_pop_message (cancellable);
 		camel_service_disconnect_sync (service, TRUE, cancellable, NULL);
 		g_free (name);
@@ -2534,20 +2523,20 @@ mapi_authenticate_sync (CamelService *service,
 	profile = camel_mapi_settings_get_profile (mapi_settings);
 
 	if (empd.krb_sso) {
-		if (e_mapi_util_trigger_krb_auth (&empd, error))
-			return CAMEL_AUTHENTICATION_ACCEPTED;
-		else
+		if (!e_mapi_util_trigger_krb_auth (&empd, error))
 			return CAMEL_AUTHENTICATION_ERROR;
-	}
 
-	password = camel_service_get_password (service);
+		password = NULL;
+	} else {
+		password = camel_service_get_password (service);
 
-	if (password == NULL) {
-		g_set_error_literal (
-			error, CAMEL_SERVICE_ERROR,
-			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
-			_("Authentication password not available"));
-		return CAMEL_AUTHENTICATION_ERROR;
+		if (password == NULL) {
+			g_set_error_literal (
+				error, CAMEL_SERVICE_ERROR,
+				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+				_("Authentication password not available"));
+			return CAMEL_AUTHENTICATION_ERROR;
+		}
 	}
 
 	password_str = g_string_new (password);
@@ -2664,6 +2653,10 @@ mapi_update_hash_table_type (CamelMapiStore *store, const gchar *full_name, guin
 	if (full_name && folder_type) {
 		if (!g_hash_table_lookup (priv->container_hash, full_name))
 			g_hash_table_insert (priv->container_hash, g_strdup (full_name), folder_type);
+		else
+			g_free (folder_type);
+	} else {
+		g_free (folder_type);
 	}
 }
 
